@@ -4,6 +4,8 @@ import asyncio
 import contextlib
 import functools
 import gc
+import inspect
+import os
 import socket
 import sys
 import unittest
@@ -56,12 +58,20 @@ else:
     SSLContext = None
 
 
+REUSE_ADDRESS = os.name == "posix" and sys.platform != "cygwin"
+
+
 def get_unused_port_socket(host: str) -> socket.socket:
     return get_port_socket(host, 0)
 
 
 def get_port_socket(host: str, port: int) -> socket.socket:
     s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    if REUSE_ADDRESS:
+        # Windows has different semantics for SO_REUSEADDR,
+        # so don't set it. Ref:
+        # https://docs.microsoft.com/en-us/windows/win32/winsock/using-so-reuseaddr-and-so-exclusiveaddruse
+        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
     s.bind((host, port))
     return s
 
@@ -69,21 +79,23 @@ def get_port_socket(host: str, port: int) -> socket.socket:
 def unused_port() -> int:
     """Return a port that is unused on the current host."""
     with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
-        s.bind(('127.0.0.1', 0))
+        s.bind(("127.0.0.1", 0))
         return s.getsockname()[1]
 
 
 class BaseTestServer(ABC):
     __test__ = False
 
-    def __init__(self,
-                 *,
-                 scheme: Union[str, object]=sentinel,
-                 loop: Optional[asyncio.AbstractEventLoop]=None,
-                 host: str='127.0.0.1',
-                 port: Optional[int]=None,
-                 skip_url_asserts: bool=False,
-                 **kwargs: Any) -> None:
+    def __init__(
+        self,
+        *,
+        scheme: Union[str, object] = sentinel,
+        loop: Optional[asyncio.AbstractEventLoop] = None,
+        host: str = "127.0.0.1",
+        port: Optional[int] = None,
+        skip_url_asserts: bool = False,
+        **kwargs: Any
+    ) -> None:
         self._loop = loop
         self.runner = None  # type: Optional[BaseRunner]
         self._root = None  # type: Optional[URL]
@@ -93,13 +105,13 @@ class BaseTestServer(ABC):
         self.scheme = scheme
         self.skip_url_asserts = skip_url_asserts
 
-    async def start_server(self,
-                           loop: Optional[asyncio.AbstractEventLoop]=None,
-                           **kwargs: Any) -> None:
+    async def start_server(
+        self, loop: Optional[asyncio.AbstractEventLoop] = None, **kwargs: Any
+    ) -> None:
         if self.runner:
             return
         self._loop = loop
-        self._ssl = kwargs.pop('ssl', None)
+        self._ssl = kwargs.pop("ssl", None)
         self.runner = await self._make_runner(**kwargs)
         await self.runner.setup()
         if not self.port:
@@ -115,13 +127,11 @@ class BaseTestServer(ABC):
         self.port = sockets[0].getsockname()[1]
         if self.scheme is sentinel:
             if self._ssl:
-                scheme = 'https'
+                scheme = "https"
             else:
-                scheme = 'http'
+                scheme = "http"
             self.scheme = scheme
-        self._root = URL('{}://{}:{}'.format(self.scheme,
-                                             self.host,
-                                             self.port))
+        self._root = URL("{}://{}:{}".format(self.scheme, self.host, self.port))
 
     @abstractmethod  # pragma: no cover
     async def _make_runner(self, **kwargs: Any) -> BaseRunner:
@@ -175,31 +185,38 @@ class BaseTestServer(ABC):
     def __enter__(self) -> None:
         raise TypeError("Use async with instead")
 
-    def __exit__(self,
-                 exc_type: Optional[Type[BaseException]],
-                 exc_value: Optional[BaseException],
-                 traceback: Optional[TracebackType]) -> None:
+    def __exit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_value: Optional[BaseException],
+        traceback: Optional[TracebackType],
+    ) -> None:
         # __exit__ should exist in pair with __enter__ but never executed
         pass  # pragma: no cover
 
-    async def __aenter__(self) -> 'BaseTestServer':
+    async def __aenter__(self) -> "BaseTestServer":
         await self.start_server(loop=self._loop)
         return self
 
-    async def __aexit__(self,
-                        exc_type: Optional[Type[BaseException]],
-                        exc_value: Optional[BaseException],
-                        traceback: Optional[TracebackType]) -> None:
+    async def __aexit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_value: Optional[BaseException],
+        traceback: Optional[TracebackType],
+    ) -> None:
         await self.close()
 
 
 class TestServer(BaseTestServer):
-
-    def __init__(self, app: Application, *,
-                 scheme: Union[str, object]=sentinel,
-                 host: str='127.0.0.1',
-                 port: Optional[int]=None,
-                 **kwargs: Any):
+    def __init__(
+        self,
+        app: Application,
+        *,
+        scheme: Union[str, object] = sentinel,
+        host: str = "127.0.0.1",
+        port: Optional[int] = None,
+        **kwargs: Any
+    ):
         self.app = app
         super().__init__(scheme=scheme, host=host, port=port, **kwargs)
 
@@ -208,20 +225,20 @@ class TestServer(BaseTestServer):
 
 
 class RawTestServer(BaseTestServer):
-
-    def __init__(self, handler: _RequestHandler, *,
-                 scheme: Union[str, object]=sentinel,
-                 host: str='127.0.0.1',
-                 port: Optional[int]=None,
-                 **kwargs: Any) -> None:
+    def __init__(
+        self,
+        handler: _RequestHandler,
+        *,
+        scheme: Union[str, object] = sentinel,
+        host: str = "127.0.0.1",
+        port: Optional[int] = None,
+        **kwargs: Any
+    ) -> None:
         self._handler = handler
         super().__init__(scheme=scheme, host=host, port=port, **kwargs)
 
-    async def _make_runner(self,
-                           debug: bool=True,
-                           **kwargs: Any) -> ServerRunner:
-        srv = Server(
-            self._handler, loop=self._loop, debug=debug, **kwargs)
+    async def _make_runner(self, debug: bool = True, **kwargs: Any) -> ServerRunner:
+        srv = Server(self._handler, loop=self._loop, debug=debug, **kwargs)
         return ServerRunner(srv, debug=debug, **kwargs)
 
 
@@ -232,22 +249,26 @@ class TestClient:
     To write functional tests for aiohttp based servers.
 
     """
+
     __test__ = False
 
-    def __init__(self, server: BaseTestServer, *,
-                 cookie_jar: Optional[AbstractCookieJar]=None,
-                 loop: Optional[asyncio.AbstractEventLoop]=None,
-                 **kwargs: Any) -> None:
+    def __init__(
+        self,
+        server: BaseTestServer,
+        *,
+        cookie_jar: Optional[AbstractCookieJar] = None,
+        loop: Optional[asyncio.AbstractEventLoop] = None,
+        **kwargs: Any
+    ) -> None:
         if not isinstance(server, BaseTestServer):
-            raise TypeError("server must be TestServer "
-                            "instance, found type: %r" % type(server))
+            raise TypeError(
+                "server must be TestServer " "instance, found type: %r" % type(server)
+            )
         self._server = server
         self._loop = loop
         if cookie_jar is None:
             cookie_jar = aiohttp.CookieJar(unsafe=True, loop=loop)
-        self._session = ClientSession(loop=loop,
-                                      cookie_jar=cookie_jar,
-                                      **kwargs)
+        self._session = ClientSession(loop=loop, cookie_jar=cookie_jar, **kwargs)
         self._closed = False
         self._responses = []  # type: List[ClientResponse]
         self._websockets = []  # type: List[ClientWebSocketResponse]
@@ -285,17 +306,13 @@ class TestClient:
     def make_url(self, path: str) -> URL:
         return self._server.make_url(path)
 
-    async def _request(self, method: str, path: str,
-                       **kwargs: Any) -> ClientResponse:
-        resp = await self._session.request(
-            method, self.make_url(path), **kwargs
-        )
+    async def _request(self, method: str, path: str, **kwargs: Any) -> ClientResponse:
+        resp = await self._session.request(method, self.make_url(path), **kwargs)
         # save it to close later
         self._responses.append(resp)
         return resp
 
-    def request(self, method: str, path: str,
-                **kwargs: Any) -> _RequestContextManager:
+    def request(self, method: str, path: str, **kwargs: Any) -> _RequestContextManager:
         """Routes a request to tested http server.
 
         The interface is identical to aiohttp.ClientSession.request,
@@ -303,51 +320,35 @@ class TestClient:
         test server.
 
         """
-        return _RequestContextManager(
-            self._request(method, path, **kwargs)
-        )
+        return _RequestContextManager(self._request(method, path, **kwargs))
 
     def get(self, path: str, **kwargs: Any) -> _RequestContextManager:
         """Perform an HTTP GET request."""
-        return _RequestContextManager(
-            self._request(hdrs.METH_GET, path, **kwargs)
-        )
+        return _RequestContextManager(self._request(hdrs.METH_GET, path, **kwargs))
 
     def post(self, path: str, **kwargs: Any) -> _RequestContextManager:
         """Perform an HTTP POST request."""
-        return _RequestContextManager(
-            self._request(hdrs.METH_POST, path, **kwargs)
-        )
+        return _RequestContextManager(self._request(hdrs.METH_POST, path, **kwargs))
 
     def options(self, path: str, **kwargs: Any) -> _RequestContextManager:
         """Perform an HTTP OPTIONS request."""
-        return _RequestContextManager(
-            self._request(hdrs.METH_OPTIONS, path, **kwargs)
-        )
+        return _RequestContextManager(self._request(hdrs.METH_OPTIONS, path, **kwargs))
 
     def head(self, path: str, **kwargs: Any) -> _RequestContextManager:
         """Perform an HTTP HEAD request."""
-        return _RequestContextManager(
-            self._request(hdrs.METH_HEAD, path, **kwargs)
-        )
+        return _RequestContextManager(self._request(hdrs.METH_HEAD, path, **kwargs))
 
     def put(self, path: str, **kwargs: Any) -> _RequestContextManager:
         """Perform an HTTP PUT request."""
-        return _RequestContextManager(
-            self._request(hdrs.METH_PUT, path, **kwargs)
-        )
+        return _RequestContextManager(self._request(hdrs.METH_PUT, path, **kwargs))
 
     def patch(self, path: str, **kwargs: Any) -> _RequestContextManager:
         """Perform an HTTP PATCH request."""
-        return _RequestContextManager(
-            self._request(hdrs.METH_PATCH, path, **kwargs)
-        )
+        return _RequestContextManager(self._request(hdrs.METH_PATCH, path, **kwargs))
 
     def delete(self, path: str, **kwargs: Any) -> _RequestContextManager:
         """Perform an HTTP PATCH request."""
-        return _RequestContextManager(
-            self._request(hdrs.METH_DELETE, path, **kwargs)
-        )
+        return _RequestContextManager(self._request(hdrs.METH_DELETE, path, **kwargs))
 
     def ws_connect(self, path: str, **kwargs: Any) -> _WSRequestContextManager:
         """Initiate websocket connection.
@@ -355,14 +356,10 @@ class TestClient:
         The api corresponds to aiohttp.ClientSession.ws_connect.
 
         """
-        return _WSRequestContextManager(
-            self._ws_connect(path, **kwargs)
-        )
+        return _WSRequestContextManager(self._ws_connect(path, **kwargs))
 
-    async def _ws_connect(self, path: str,
-                          **kwargs: Any) -> ClientWebSocketResponse:
-        ws = await self._session.ws_connect(
-            self.make_url(path), **kwargs)
+    async def _ws_connect(self, path: str, **kwargs: Any) -> ClientWebSocketResponse:
+        ws = await self._session.ws_connect(self.make_url(path), **kwargs)
         self._websockets.append(ws)
         return ws
 
@@ -390,21 +387,25 @@ class TestClient:
     def __enter__(self) -> None:
         raise TypeError("Use async with instead")
 
-    def __exit__(self,
-                 exc_type: Optional[Type[BaseException]],
-                 exc: Optional[BaseException],
-                 tb: Optional[TracebackType]) -> None:
+    def __exit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc: Optional[BaseException],
+        tb: Optional[TracebackType],
+    ) -> None:
         # __exit__ should exist in pair with __enter__ but never executed
         pass  # pragma: no cover
 
-    async def __aenter__(self) -> 'TestClient':
+    async def __aenter__(self) -> "TestClient":
         await self.start_server()
         return self
 
-    async def __aexit__(self,
-                        exc_type: Optional[Type[BaseException]],
-                        exc: Optional[BaseException],
-                        tb: Optional[TracebackType]) -> None:
+    async def __aexit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc: Optional[BaseException],
+        tb: Optional[TracebackType],
+    ) -> None:
         await self.close()
 
 
@@ -446,8 +447,7 @@ class AioHTTPTestCase(unittest.TestCase):
 
         self.app = self.loop.run_until_complete(self.get_application())
         self.server = self.loop.run_until_complete(self.get_server(self.app))
-        self.client = self.loop.run_until_complete(
-            self.get_client(self.server))
+        self.client = self.loop.run_until_complete(self.get_client(self.server))
 
         self.loop.run_until_complete(self.client.start_server())
 
@@ -483,8 +483,7 @@ def unittest_run_loop(func: Any, *args: Any, **kwargs: Any) -> Any:
 
     @functools.wraps(func, *args, **kwargs)
     def new_func(self: Any, *inner_args: Any, **inner_kwargs: Any) -> Any:
-        return self.loop.run_until_complete(
-            func(self, *inner_args, **inner_kwargs))
+        return self.loop.run_until_complete(func(self, *inner_args, **inner_kwargs))
 
     return new_func
 
@@ -493,8 +492,9 @@ _LOOP_FACTORY = Callable[[], asyncio.AbstractEventLoop]
 
 
 @contextlib.contextmanager
-def loop_context(loop_factory: _LOOP_FACTORY=asyncio.new_event_loop,
-                 fast: bool=False) -> Iterator[asyncio.AbstractEventLoop]:
+def loop_context(
+    loop_factory: _LOOP_FACTORY = asyncio.new_event_loop, fast: bool = False
+) -> Iterator[asyncio.AbstractEventLoop]:
     """A contextmanager that creates an event_loop, for test purposes.
 
     Handles the creation and cleanup of a test loop.
@@ -505,7 +505,7 @@ def loop_context(loop_factory: _LOOP_FACTORY=asyncio.new_event_loop,
 
 
 def setup_test_loop(
-        loop_factory: _LOOP_FACTORY=asyncio.new_event_loop
+    loop_factory: _LOOP_FACTORY = asyncio.new_event_loop,
 ) -> asyncio.AbstractEventLoop:
     """Create and return an asyncio.BaseEventLoop
     instance.
@@ -516,22 +516,21 @@ def setup_test_loop(
     loop = loop_factory()
     try:
         module = loop.__class__.__module__
-        skip_watcher = 'uvloop' in module
+        skip_watcher = "uvloop" in module
     except AttributeError:  # pragma: no cover
         # Just in case
         skip_watcher = True
     asyncio.set_event_loop(loop)
     if sys.platform != "win32" and not skip_watcher:
         policy = asyncio.get_event_loop_policy()
-        watcher = asyncio.SafeChildWatcher()  # type: ignore
+        watcher = asyncio.SafeChildWatcher()
         watcher.attach_loop(loop)
         with contextlib.suppress(NotImplementedError):
             policy.set_child_watcher(watcher)
     return loop
 
 
-def teardown_test_loop(loop: asyncio.AbstractEventLoop,
-                       fast: bool=False) -> None:
+def teardown_test_loop(loop: asyncio.AbstractEventLoop, fast: bool = False) -> None:
     """Teardown and cleanup an event_loop created
     by setup_test_loop.
 
@@ -566,11 +565,11 @@ def _create_app_mock() -> mock.MagicMock:
     return app
 
 
-def _create_transport(sslcontext: Optional[SSLContext]=None) -> mock.Mock:
+def _create_transport(sslcontext: Optional[SSLContext] = None) -> mock.Mock:
     transport = mock.Mock()
 
     def get_extra_info(key: str) -> Optional[SSLContext]:
-        if key == 'sslcontext':
+        if key == "sslcontext":
             return sslcontext
         else:
             return None
@@ -579,19 +578,23 @@ def _create_transport(sslcontext: Optional[SSLContext]=None) -> mock.Mock:
     return transport
 
 
-def make_mocked_request(method: str, path: str,
-                        headers: Any=None, *,
-                        match_info: Any=sentinel,
-                        version: HttpVersion=HttpVersion(1, 1),
-                        closing: bool=False,
-                        app: Any=None,
-                        writer: Any=sentinel,
-                        protocol: Any=sentinel,
-                        transport: Any=sentinel,
-                        payload: Any=sentinel,
-                        sslcontext: Optional[SSLContext]=None,
-                        client_max_size: int=1024**2,
-                        loop: Any=...) -> Any:
+def make_mocked_request(
+    method: str,
+    path: str,
+    headers: Any = None,
+    *,
+    match_info: Any = sentinel,
+    version: HttpVersion = HttpVersion(1, 1),
+    closing: bool = False,
+    app: Any = None,
+    writer: Any = sentinel,
+    protocol: Any = sentinel,
+    transport: Any = sentinel,
+    payload: Any = sentinel,
+    sslcontext: Optional[SSLContext] = None,
+    client_max_size: int = 1024 ** 2,
+    loop: Any = ...
+) -> Any:
     """Creates mocked web.Request testing purposes.
 
     Useful in unit tests, when spinning full web server is overkill or
@@ -610,16 +613,26 @@ def make_mocked_request(method: str, path: str,
     if headers:
         headers = CIMultiDictProxy(CIMultiDict(headers))
         raw_hdrs = tuple(
-            (k.encode('utf-8'), v.encode('utf-8')) for k, v in headers.items())
+            (k.encode("utf-8"), v.encode("utf-8")) for k, v in headers.items()
+        )
     else:
         headers = CIMultiDictProxy(CIMultiDict())
         raw_hdrs = ()
 
-    chunked = 'chunked' in headers.get(hdrs.TRANSFER_ENCODING, '').lower()
+    chunked = "chunked" in headers.get(hdrs.TRANSFER_ENCODING, "").lower()
 
     message = RawRequestMessage(
-        method, path, version, headers,
-        raw_hdrs, closing, False, False, chunked, URL(path))
+        method,
+        path,
+        version,
+        headers,
+        raw_hdrs,
+        closing,
+        False,
+        False,
+        chunked,
+        URL(path),
+    )
     if app is None:
         app = _create_app_mock()
 
@@ -644,25 +657,29 @@ def make_mocked_request(method: str, path: str,
     if payload is sentinel:
         payload = mock.Mock()
 
-    req = Request(message, payload,
-                  protocol, writer, task, loop,
-                  client_max_size=client_max_size)
+    req = Request(
+        message, payload, protocol, writer, task, loop, client_max_size=client_max_size
+    )
 
     match_info = UrlMappingMatchInfo(
-        {} if match_info is sentinel else match_info, mock.Mock())
+        {} if match_info is sentinel else match_info, mock.Mock()
+    )
     match_info.add_app(app)
     req._match_info = match_info
 
     return req
 
 
-def make_mocked_coro(return_value: Any=sentinel,
-                     raise_exception: Any=sentinel) -> Any:
+def make_mocked_coro(
+    return_value: Any = sentinel, raise_exception: Any = sentinel
+) -> Any:
     """Creates a coroutine mock."""
-    @asyncio.coroutine
-    def mock_coro(*args: Any, **kwargs: Any) -> Any:
+
+    async def mock_coro(*args: Any, **kwargs: Any) -> Any:
         if raise_exception is not sentinel:
             raise raise_exception
-        return return_value
+        if not inspect.isawaitable(return_value):
+            return return_value
+        await return_value
 
     return mock.Mock(wraps=mock_coro)
