@@ -24,14 +24,13 @@ from .log import server_logger
 from .typedefs import LooseHeaders
 from .web_exceptions import (
     HTTPNotModified,
-    HTTPOk,
     HTTPPartialContent,
     HTTPPreconditionFailed,
     HTTPRequestRangeNotSatisfiable,
 )
 from .web_response import StreamResponse
 
-__all__ = ('FileResponse',)
+__all__ = ("FileResponse",)
 
 if TYPE_CHECKING:  # pragma: no cover
     from .web_request import BaseRequest  # noqa
@@ -44,18 +43,20 @@ NOSENDFILE = bool(os.environ.get("AIOHTTP_NOSENDFILE"))
 
 
 class SendfileStreamWriter(StreamWriter):
-
-    def __init__(self,
-                 protocol: BaseProtocol,
-                 loop: asyncio.AbstractEventLoop,
-                 fobj: IO[Any],
-                 count: int,
-                 on_chunk_sent: _T_OnChunkSent=None) -> None:
+    def __init__(
+        self,
+        protocol: BaseProtocol,
+        loop: asyncio.AbstractEventLoop,
+        fobj: IO[Any],
+        offset: int,
+        count: int,
+        on_chunk_sent: _T_OnChunkSent = None,
+    ) -> None:
         super().__init__(protocol, loop, on_chunk_sent)
         self._sendfile_buffer = []  # type: List[bytes]
         self._fobj = fobj
         self._count = count
-        self._offset = fobj.tell()
+        self._offset = offset
         self._in_fd = fobj.fileno()
 
     def _write(self, chunk: bytes) -> None:
@@ -65,7 +66,7 @@ class SendfileStreamWriter(StreamWriter):
         self.output_size += len(chunk)
         self._sendfile_buffer.append(chunk)
 
-    def _sendfile_cb(self, fut: 'asyncio.Future[None]', out_fd: int) -> None:
+    def _sendfile_cb(self, fut: "asyncio.Future[None]", out_fd: int) -> None:
         if fut.cancelled():
             return
         try:
@@ -76,10 +77,7 @@ class SendfileStreamWriter(StreamWriter):
 
     def _do_sendfile(self, out_fd: int) -> bool:
         try:
-            n = os.sendfile(out_fd,
-                            self._in_fd,
-                            self._offset,
-                            self._count)
+            n = os.sendfile(out_fd, self._in_fd, self._offset, self._count)
             if n == 0:  # in_fd EOF reached
                 n = self._count
         except (BlockingIOError, InterruptedError):
@@ -90,17 +88,31 @@ class SendfileStreamWriter(StreamWriter):
         assert self._count >= 0
         return self._count == 0
 
-    def _done_fut(self, out_fd: int, fut: 'asyncio.Future[None]') -> None:
+    def _done_fut(self, out_fd: int, fut: "asyncio.Future[None]") -> None:
         self.loop.remove_writer(out_fd)
 
     async def sendfile(self) -> None:
         assert self.transport is not None
-        out_socket = self.transport.get_extra_info('socket').dup()
+        loop = self.loop
+        data = b"".join(self._sendfile_buffer)
+        if hasattr(loop, "sendfile"):
+            # Python 3.7+
+            self.transport.write(data)
+            if self._count != 0:
+                await loop.sendfile(
+                    self.transport,
+                    self._fobj,
+                    self._offset,
+                    self._count
+                )
+            await super().write_eof()
+            return
+
+        self._fobj.seek(self._offset)
+        out_socket = self.transport.get_extra_info("socket").dup()
         out_socket.setblocking(False)
         out_fd = out_socket.fileno()
 
-        loop = self.loop
-        data = b''.join(self._sendfile_buffer)
         try:
             await loop.sock_sendall(out_socket, data)
             if not self._do_sendfile(out_fd):
@@ -111,25 +123,28 @@ class SendfileStreamWriter(StreamWriter):
         except asyncio.CancelledError:
             raise
         except Exception:
-            server_logger.debug('Socket error')
+            server_logger.debug("Socket error")
             self.transport.close()
         finally:
             out_socket.close()
 
         await super().write_eof()
 
-    async def write_eof(self, chunk: bytes=b'') -> None:
+    async def write_eof(self, chunk: bytes = b"") -> None:
         pass
 
 
 class FileResponse(StreamResponse):
     """A response object can be used to send files."""
 
-    def __init__(self, path: Union[str, pathlib.Path],
-                 chunk_size: int=256*1024,
-                 status: int=200,
-                 reason: Optional[str]=None,
-                 headers: Optional[LooseHeaders]=None) -> None:
+    def __init__(
+        self,
+        path: Union[str, pathlib.Path],
+        chunk_size: int = 256 * 1024,
+        status: int = 200,
+        reason: Optional[str] = None,
+        headers: Optional[LooseHeaders] = None,
+    ) -> None:
         super().__init__(status=status, reason=reason, headers=headers)
 
         if isinstance(path, str):
@@ -138,9 +153,9 @@ class FileResponse(StreamResponse):
         self._path = path
         self._chunk_size = chunk_size
 
-    async def _sendfile_system(self, request: 'BaseRequest',
-                               fobj: IO[Any],
-                               count: int) -> AbstractStreamWriter:
+    async def _sendfile_system(
+        self, request: "BaseRequest", fobj: IO[Any], offset: int, count: int
+    ) -> AbstractStreamWriter:
         # Write count bytes of fobj to resp using
         # the os.sendfile system call.
         #
@@ -154,16 +169,15 @@ class FileResponse(StreamResponse):
 
         transport = request.transport
         assert transport is not None
-        if (transport.get_extra_info("sslcontext") or
-                transport.get_extra_info("socket") is None or
-                self.compression):
-            writer = await self._sendfile_fallback(request, fobj, count)
+        if (
+            transport.get_extra_info("sslcontext")
+            or transport.get_extra_info("socket") is None
+            or self.compression
+        ):
+            writer = await self._sendfile_fallback(request, fobj, offset, count)
         else:
             writer = SendfileStreamWriter(
-                request.protocol,
-                request._loop,
-                fobj,
-                count
+                request.protocol, request._loop, fobj, offset, count
             )
             request._payload_writer = writer
 
@@ -172,9 +186,9 @@ class FileResponse(StreamResponse):
 
         return writer
 
-    async def _sendfile_fallback(self, request: 'BaseRequest',
-                                 fobj: IO[Any],
-                                 count: int) -> AbstractStreamWriter:
+    async def _sendfile_fallback(
+        self, request: "BaseRequest", fobj: IO[Any], offset: int, count: int
+    ) -> AbstractStreamWriter:
         # Mimic the _sendfile_system() method, but without using the
         # os.sendfile() system call. This should be used on systems
         # that don't support the os.sendfile().
@@ -188,15 +202,15 @@ class FileResponse(StreamResponse):
         chunk_size = self._chunk_size
         loop = asyncio.get_event_loop()
 
+        await loop.run_in_executor(None, fobj.seek, offset)
+
         chunk = await loop.run_in_executor(None, fobj.read, chunk_size)
         while chunk:
             await writer.write(chunk)
             count = count - chunk_size
             if count <= 0:
                 break
-            chunk = await loop.run_in_executor(
-                None, fobj.read, min(chunk_size, count)
-            )
+            chunk = await loop.run_in_executor(None, fobj.read, min(chunk_size, count))
 
         await writer.drain()
         return writer
@@ -206,15 +220,12 @@ class FileResponse(StreamResponse):
     else:  # pragma: no cover
         _sendfile = _sendfile_fallback
 
-    async def prepare(
-            self,
-            request: 'BaseRequest'
-    ) -> Optional[AbstractStreamWriter]:
+    async def prepare(self, request: "BaseRequest") -> Optional[AbstractStreamWriter]:
         filepath = self._path
 
         gzip = False
-        if 'gzip' in request.headers.get(hdrs.ACCEPT_ENCODING, ''):
-            gzip_path = filepath.with_name(filepath.name + '.gz')
+        if "gzip" in request.headers.get(hdrs.ACCEPT_ENCODING, ""):
+            gzip_path = filepath.with_name(filepath.name + ".gz")
 
             if gzip_path.is_file():
                 filepath = gzip_path
@@ -239,13 +250,13 @@ class FileResponse(StreamResponse):
         if hdrs.CONTENT_TYPE not in self.headers:
             ct, encoding = mimetypes.guess_type(str(filepath))
             if not ct:
-                ct = 'application/octet-stream'
+                ct = "application/octet-stream"
             should_set_ct = True
         else:
-            encoding = 'gzip' if gzip else None
+            encoding = "gzip" if gzip else None
             should_set_ct = False
 
-        status = HTTPOk.status_code
+        status = self._status
         file_size = st.st_size
         count = file_size
 
@@ -274,8 +285,7 @@ class FileResponse(StreamResponse):
                 #
                 # Will do the same below. Many servers ignore this and do not
                 # send a Content-Range header with HTTP 416
-                self.headers[hdrs.CONTENT_RANGE] = 'bytes */{0}'.format(
-                    file_size)
+                self.headers[hdrs.CONTENT_RANGE] = "bytes */{0}".format(file_size)
                 self.set_status(HTTPRequestRangeNotSatisfiable.status_code)
                 return await super().prepare(request)
 
@@ -297,8 +307,9 @@ class FileResponse(StreamResponse):
                     # of the representation (i.e., the server replaces the
                     # value of last-byte-pos with a value that is one less than
                     # the current length of the selected representation).
-                    count = min(end if end is not None else file_size,
-                                file_size) - start
+                    count = (
+                        min(end if end is not None else file_size, file_size) - start
+                    )
 
                 if start >= file_size:
                     # HTTP 416 should be returned in this case.
@@ -310,16 +321,15 @@ class FileResponse(StreamResponse):
                     # suffix-byte-range-spec with a non-zero suffix-length,
                     # then the byte-range-set is satisfiable. Otherwise, the
                     # byte-range-set is unsatisfiable.
-                    self.headers[hdrs.CONTENT_RANGE] = 'bytes */{0}'.format(
-                        file_size)
+                    self.headers[hdrs.CONTENT_RANGE] = "bytes */{0}".format(file_size)
                     self.set_status(HTTPRequestRangeNotSatisfiable.status_code)
                     return await super().prepare(request)
 
                 status = HTTPPartialContent.status_code
                 # Even though you are sending the whole file, you should still
                 # return a HTTP 206 for a Range request.
+                self.set_status(status)
 
-        self.set_status(status)
         if should_set_ct:
             self.content_type = ct  # type: ignore
         if encoding:
@@ -329,19 +339,25 @@ class FileResponse(StreamResponse):
         self.last_modified = st.st_mtime  # type: ignore
         self.content_length = count
 
-        self.headers[hdrs.ACCEPT_RANGES] = 'bytes'
+        self.headers[hdrs.ACCEPT_RANGES] = "bytes"
 
         real_start = cast(int, start)
 
         if status == HTTPPartialContent.status_code:
-            self.headers[hdrs.CONTENT_RANGE] = 'bytes {0}-{1}/{2}'.format(
-                real_start, real_start + count - 1, file_size)
+            self.headers[hdrs.CONTENT_RANGE] = "bytes {0}-{1}/{2}".format(
+                real_start, real_start + count - 1, file_size
+            )
 
-        fobj = await loop.run_in_executor(None, filepath.open, 'rb')
+        if request.method == hdrs.METH_HEAD or self.status in [204, 304]:
+            return await super().prepare(request)
+
+        fobj = await loop.run_in_executor(None, filepath.open, "rb")
         if start:  # be aware that start could be None or int=0 here.
-            await loop.run_in_executor(None, fobj.seek, start)
+            offset = start
+        else:
+            offset = 0
 
         try:
-            return await self._sendfile(request, fobj, count)
+            return await self._sendfile(request, fobj, offset, count)
         finally:
             await loop.run_in_executor(None, fobj.close)
