@@ -3,6 +3,7 @@ import contextlib
 import gc
 import json
 import re
+import sys
 from http.cookies import SimpleCookie
 from io import BytesIO
 from unittest import mock
@@ -17,15 +18,17 @@ from aiohttp.client import ClientSession
 from aiohttp.client_reqrep import ClientRequest
 from aiohttp.connector import BaseConnector, TCPConnector
 from aiohttp.helpers import DEBUG, PY_36
+from aiohttp.test_utils import make_mocked_coro
 
 
 @pytest.fixture
 def connector(loop):
     async def make_conn():
         return BaseConnector(loop=loop)
+
     conn = loop.run_until_complete(make_conn())
     proto = mock.Mock()
-    conn._conns['a'] = [(proto, 123)]
+    conn._conns["a"] = [(proto, 123)]
     yield conn
     conn.close()
 
@@ -38,6 +41,7 @@ def create_session(loop):
         nonlocal session
         session = ClientSession(*args, loop=loop, **kwargs)
         return session
+
     yield maker
     if session is not None:
         loop.run_until_complete(session.close())
@@ -58,7 +62,8 @@ def params():
         compress="deflate",
         chunked=True,
         expect100=True,
-        read_until_eof=False)
+        read_until_eof=False,
+    )
 
 
 async def test_close_coro(create_session) -> None:
@@ -67,66 +72,57 @@ async def test_close_coro(create_session) -> None:
 
 
 async def test_init_headers_simple_dict(create_session) -> None:
-    session = await create_session(headers={"h1": "header1",
-                                            "h2": "header2"})
-    assert (sorted(session._default_headers.items()) ==
-            ([("h1", "header1"), ("h2", "header2")]))
+    session = await create_session(headers={"h1": "header1", "h2": "header2"})
+    assert sorted(session.headers.items()) == ([("h1", "header1"), ("h2", "header2")])
 
 
 async def test_init_headers_list_of_tuples(create_session) -> None:
-    session = await create_session(headers=[("h1", "header1"),
-                                            ("h2", "header2"),
-                                            ("h3", "header3")])
-    assert (session._default_headers ==
-            CIMultiDict([("h1", "header1"),
-                         ("h2", "header2"),
-                         ("h3", "header3")]))
+    session = await create_session(
+        headers=[("h1", "header1"), ("h2", "header2"), ("h3", "header3")]
+    )
+    assert session.headers == CIMultiDict(
+        [("h1", "header1"), ("h2", "header2"), ("h3", "header3")]
+    )
 
 
 async def test_init_headers_MultiDict(create_session) -> None:
-    session = await create_session(headers=MultiDict([("h1", "header1"),
-                                                      ("h2", "header2"),
-                                                      ("h3", "header3")]))
-    assert (session._default_headers ==
-            CIMultiDict([("H1", "header1"),
-                         ("H2", "header2"),
-                         ("H3", "header3")]))
+    session = await create_session(
+        headers=MultiDict([("h1", "header1"), ("h2", "header2"), ("h3", "header3")])
+    )
+    assert session.headers == CIMultiDict(
+        [("H1", "header1"), ("H2", "header2"), ("H3", "header3")]
+    )
 
 
-async def test_init_headers_list_of_tuples_with_duplicates(
-        create_session) -> None:
-    session = await create_session(headers=[("h1", "header11"),
-                                            ("h2", "header21"),
-                                            ("h1", "header12")])
-    assert (session._default_headers ==
-            CIMultiDict([("H1", "header11"),
-                         ("H2", "header21"),
-                         ("H1", "header12")]))
+async def test_init_headers_list_of_tuples_with_duplicates(create_session) -> None:
+    session = await create_session(
+        headers=[("h1", "header11"), ("h2", "header21"), ("h1", "header12")]
+    )
+    assert session.headers == CIMultiDict(
+        [("H1", "header11"), ("H2", "header21"), ("H1", "header12")]
+    )
 
 
 async def test_init_cookies_with_simple_dict(create_session) -> None:
-    session = await create_session(cookies={"c1": "cookie1",
-                                            "c2": "cookie2"})
+    session = await create_session(cookies={"c1": "cookie1", "c2": "cookie2"})
     cookies = session.cookie_jar.filter_cookies()
-    assert set(cookies) == {'c1', 'c2'}
-    assert cookies['c1'].value == 'cookie1'
-    assert cookies['c2'].value == 'cookie2'
+    assert set(cookies) == {"c1", "c2"}
+    assert cookies["c1"].value == "cookie1"
+    assert cookies["c2"].value == "cookie2"
 
 
 async def test_init_cookies_with_list_of_tuples(create_session) -> None:
-    session = await create_session(cookies=[("c1", "cookie1"),
-                                            ("c2", "cookie2")])
+    session = await create_session(cookies=[("c1", "cookie1"), ("c2", "cookie2")])
 
     cookies = session.cookie_jar.filter_cookies()
-    assert set(cookies) == {'c1', 'c2'}
-    assert cookies['c1'].value == 'cookie1'
-    assert cookies['c2'].value == 'cookie2'
+    assert set(cookies) == {"c1", "c2"}
+    assert cookies["c1"].value == "cookie1"
+    assert cookies["c2"].value == "cookie2"
 
 
 async def test_merge_headers(create_session) -> None:
     # Check incoming simple dict
-    session = await create_session(headers={"h1": "header1",
-                                            "h2": "header2"})
+    session = await create_session(headers={"h1": "header1", "h2": "header2"})
     headers = session._prepare_headers({"h1": "h1"})
 
     assert isinstance(headers, CIMultiDict)
@@ -134,127 +130,144 @@ async def test_merge_headers(create_session) -> None:
 
 
 async def test_merge_headers_with_multi_dict(create_session) -> None:
-    session = await create_session(headers={"h1": "header1",
-                                            "h2": "header2"})
+    session = await create_session(headers={"h1": "header1", "h2": "header2"})
     headers = session._prepare_headers(MultiDict([("h1", "h1")]))
     assert isinstance(headers, CIMultiDict)
     assert headers == {"h1": "h1", "h2": "header2"}
 
 
 async def test_merge_headers_with_list_of_tuples(create_session) -> None:
-    session = await create_session(headers={"h1": "header1",
-                                            "h2": "header2"})
+    session = await create_session(headers={"h1": "header1", "h2": "header2"})
     headers = session._prepare_headers([("h1", "h1")])
     assert isinstance(headers, CIMultiDict)
     assert headers == {"h1": "h1", "h2": "header2"}
 
 
 async def test_merge_headers_with_list_of_tuples_duplicated_names(
-        create_session) -> None:
-    session = await create_session(headers={"h1": "header1",
-                                            "h2": "header2"})
+    create_session,
+) -> None:
+    session = await create_session(headers={"h1": "header1", "h2": "header2"})
 
-    headers = session._prepare_headers([("h1", "v1"),
-                                        ("h1", "v2")])
+    headers = session._prepare_headers([("h1", "v1"), ("h1", "v2")])
 
     assert isinstance(headers, CIMultiDict)
-    assert list(sorted(headers.items())) == [("h1", "v1"),
-                                             ("h1", "v2"),
-                                             ("h2", "header2")]
+    assert list(sorted(headers.items())) == [
+        ("h1", "v1"),
+        ("h1", "v2"),
+        ("h2", "header2"),
+    ]
 
 
 def test_http_GET(session, params) -> None:
-    with mock.patch("aiohttp.client.ClientSession._request") as patched:
-        session.get("http://test.example.com",
-                    params={"x": 1},
-                    **params)
+    # Python 3.8 will auto use mock.AsyncMock, it has different behavior
+    with mock.patch(
+        "aiohttp.client.ClientSession._request", new_callable=mock.MagicMock
+    ) as patched:
+        session.get("http://test.example.com", params={"x": 1}, **params)
     assert patched.called, "`ClientSession._request` not called"
-    assert list(patched.call_args) == [("GET", "http://test.example.com",),
-                                       dict(
-                                           params={"x": 1},
-                                           allow_redirects=True,
-                                           **params)]
+    assert list(patched.call_args) == [
+        (
+            "GET",
+            "http://test.example.com",
+        ),
+        dict(params={"x": 1}, allow_redirects=True, **params),
+    ]
 
 
 def test_http_OPTIONS(session, params) -> None:
-    with mock.patch("aiohttp.client.ClientSession._request") as patched:
-        session.options("http://opt.example.com",
-                        params={"x": 2},
-                        **params)
+    with mock.patch(
+        "aiohttp.client.ClientSession._request", new_callable=mock.MagicMock
+    ) as patched:
+        session.options("http://opt.example.com", params={"x": 2}, **params)
     assert patched.called, "`ClientSession._request` not called"
-    assert list(patched.call_args) == [("OPTIONS", "http://opt.example.com",),
-                                       dict(
-                                           params={"x": 2},
-                                           allow_redirects=True,
-                                           **params)]
+    assert list(patched.call_args) == [
+        (
+            "OPTIONS",
+            "http://opt.example.com",
+        ),
+        dict(params={"x": 2}, allow_redirects=True, **params),
+    ]
 
 
 def test_http_HEAD(session, params) -> None:
-    with mock.patch("aiohttp.client.ClientSession._request") as patched:
-        session.head("http://head.example.com",
-                     params={"x": 2},
-                     **params)
+    with mock.patch(
+        "aiohttp.client.ClientSession._request", new_callable=mock.MagicMock
+    ) as patched:
+        session.head("http://head.example.com", params={"x": 2}, **params)
     assert patched.called, "`ClientSession._request` not called"
-    assert list(patched.call_args) == [("HEAD", "http://head.example.com",),
-                                       dict(
-                                           params={"x": 2},
-                                           allow_redirects=False,
-                                           **params)]
+    assert list(patched.call_args) == [
+        (
+            "HEAD",
+            "http://head.example.com",
+        ),
+        dict(params={"x": 2}, allow_redirects=False, **params),
+    ]
 
 
 def test_http_POST(session, params) -> None:
-    with mock.patch("aiohttp.client.ClientSession._request") as patched:
-        session.post("http://post.example.com",
-                     params={"x": 2},
-                     data="Some_data",
-                     **params)
+    with mock.patch(
+        "aiohttp.client.ClientSession._request", new_callable=mock.MagicMock
+    ) as patched:
+        session.post(
+            "http://post.example.com", params={"x": 2}, data="Some_data", **params
+        )
     assert patched.called, "`ClientSession._request` not called"
-    assert list(patched.call_args) == [("POST", "http://post.example.com",),
-                                       dict(
-                                           params={"x": 2},
-                                           data="Some_data",
-                                           **params)]
+    assert list(patched.call_args) == [
+        (
+            "POST",
+            "http://post.example.com",
+        ),
+        dict(params={"x": 2}, data="Some_data", **params),
+    ]
 
 
 def test_http_PUT(session, params) -> None:
-    with mock.patch("aiohttp.client.ClientSession._request") as patched:
-        session.put("http://put.example.com",
-                    params={"x": 2},
-                    data="Some_data",
-                    **params)
+    with mock.patch(
+        "aiohttp.client.ClientSession._request", new_callable=mock.MagicMock
+    ) as patched:
+        session.put(
+            "http://put.example.com", params={"x": 2}, data="Some_data", **params
+        )
     assert patched.called, "`ClientSession._request` not called"
-    assert list(patched.call_args) == [("PUT", "http://put.example.com",),
-                                       dict(
-                                           params={"x": 2},
-                                           data="Some_data",
-                                           **params)]
+    assert list(patched.call_args) == [
+        (
+            "PUT",
+            "http://put.example.com",
+        ),
+        dict(params={"x": 2}, data="Some_data", **params),
+    ]
 
 
 def test_http_PATCH(session, params) -> None:
-    with mock.patch("aiohttp.client.ClientSession._request") as patched:
-        session.patch("http://patch.example.com",
-                      params={"x": 2},
-                      data="Some_data",
-                      **params)
+    with mock.patch(
+        "aiohttp.client.ClientSession._request", new_callable=mock.MagicMock
+    ) as patched:
+        session.patch(
+            "http://patch.example.com", params={"x": 2}, data="Some_data", **params
+        )
     assert patched.called, "`ClientSession._request` not called"
-    assert list(patched.call_args) == [("PATCH", "http://patch.example.com",),
-                                       dict(
-                                           params={"x": 2},
-                                           data="Some_data",
-                                           **params)]
+    assert list(patched.call_args) == [
+        (
+            "PATCH",
+            "http://patch.example.com",
+        ),
+        dict(params={"x": 2}, data="Some_data", **params),
+    ]
 
 
 def test_http_DELETE(session, params) -> None:
-    with mock.patch("aiohttp.client.ClientSession._request") as patched:
-        session.delete("http://delete.example.com",
-                       params={"x": 2},
-                       **params)
+    with mock.patch(
+        "aiohttp.client.ClientSession._request", new_callable=mock.MagicMock
+    ) as patched:
+        session.delete("http://delete.example.com", params={"x": 2}, **params)
     assert patched.called, "`ClientSession._request` not called"
-    assert list(patched.call_args) == [("DELETE",
-                                        "http://delete.example.com",),
-                                       dict(
-                                           params={"x": 2},
-                                           **params)]
+    assert list(patched.call_args) == [
+        (
+            "DELETE",
+            "http://delete.example.com",
+        ),
+        dict(params={"x": 2}, **params),
+    ]
 
 
 async def test_close(create_session, connector) -> None:
@@ -273,7 +286,7 @@ async def test_closed(session) -> None:
 
 async def test_connector(create_session, loop, mocker) -> None:
     connector = TCPConnector(loop=loop)
-    mocker.spy(connector, 'close')
+    mocker.spy(connector, "close")
     session = await create_session(connector=connector)
     assert session.connector is connector
 
@@ -285,7 +298,7 @@ async def test_connector(create_session, loop, mocker) -> None:
 async def test_create_connector(create_session, loop, mocker) -> None:
     session = await create_session()
     connector = session.connector
-    mocker.spy(session.connector, 'close')
+    mocker.spy(session.connector, "close")
 
     await session.close()
     assert connector.close.called
@@ -298,15 +311,19 @@ def test_connector_loop(loop) -> None:
 
         async def make_connector():
             return TCPConnector()
+
         connector = another_loop.run_until_complete(make_connector())
 
         stack.enter_context(contextlib.closing(connector))
         with pytest.raises(RuntimeError) as ctx:
+
             async def make_sess():
                 return ClientSession(connector=connector, loop=loop)
+
             loop.run_until_complete(make_sess())
-        assert re.match("Session and connector has to use same event loop",
-                        str(ctx.value))
+        assert re.match(
+            "Session and connector has to use same event loop", str(ctx.value)
+        )
 
 
 def test_detach(session) -> None:
@@ -324,7 +341,7 @@ def test_detach(session) -> None:
 async def test_request_closed_session(session) -> None:
     await session.close()
     with pytest.raises(RuntimeError):
-        await session.request('get', '/')
+        await session.request("get", "/")
 
 
 def test_close_flag_for_closed_connector(session) -> None:
@@ -356,8 +373,7 @@ async def test_del(connector, loop) -> None:
         gc.collect()
 
     assert len(logs) == 1
-    expected = {'client_session': mock.ANY,
-                'message': 'Unclosed client session'}
+    expected = {"client_session": mock.ANY, "message": "Unclosed client session"}
     assert logs[0] == expected
 
 
@@ -373,9 +389,11 @@ async def test_del_debug(connector, loop) -> None:
         gc.collect()
 
     assert len(logs) == 1
-    expected = {'client_session': mock.ANY,
-                'message': 'Unclosed client session',
-                'source_traceback': mock.ANY}
+    expected = {
+        "client_session": mock.ANY,
+        "message": "Unclosed client session",
+        "source_traceback": mock.ANY,
+    }
     assert logs[0] == expected
 
 
@@ -405,11 +423,12 @@ async def test_reraise_os_error(create_session) -> None:
     async def create_connection(req, traces, timeout):
         # return self.transport, self.protocol
         return mock.Mock()
+
     session._connector._create_connection = create_connection
     session._connector._release = mock.Mock()
 
     with pytest.raises(aiohttp.ClientOSError) as ctx:
-        await session.request('get', 'http://example.com')
+        await session.request("get", "http://example.com")
     e = ctx.value
     assert e.errno == err.errno
     assert e.strerror == err.strerror
@@ -443,7 +462,7 @@ async def test_close_conn_on_error(create_session) -> None:
     session._connector._release = mock.Mock()
 
     with pytest.raises(UnexpectedException):
-        async with session.request('get', 'http://example.com') as resp:
+        async with session.request("get", "http://example.com") as resp:
             await resp.text()
 
     # normally called during garbage collection.  triggers an exception
@@ -467,11 +486,9 @@ async def test_cookie_jar_usage(loop, aiohttp_client) -> None:
         return resp
 
     app = web.Application()
-    app.router.add_route('GET', '/', handler)
+    app.router.add_route("GET", "/", handler)
     session = await aiohttp_client(
-        app,
-        cookies={"request": "req_value"},
-        cookie_jar=jar
+        app, cookies={"request": "req_value"}, cookie_jar=jar
     )
 
     # Updating the cookie jar with initial user defined cookies
@@ -506,33 +523,35 @@ async def test_session_loop(loop) -> None:
 
 
 def test_proxy_str(session, params) -> None:
-    with mock.patch("aiohttp.client.ClientSession._request") as patched:
-        session.get("http://test.example.com",
-                    proxy='http://proxy.com',
-                    **params)
+    with mock.patch(
+        "aiohttp.client.ClientSession._request", new_callable=mock.MagicMock
+    ) as patched:
+        session.get("http://test.example.com", proxy="http://proxy.com", **params)
     assert patched.called, "`ClientSession._request` not called"
-    assert list(patched.call_args) == [("GET", "http://test.example.com",),
-                                       dict(
-                                           allow_redirects=True,
-                                           proxy='http://proxy.com',
-                                           **params)]
+    assert list(patched.call_args) == [
+        (
+            "GET",
+            "http://test.example.com",
+        ),
+        dict(allow_redirects=True, proxy="http://proxy.com", **params),
+    ]
 
 
 async def test_request_tracing(loop, aiohttp_client) -> None:
     async def handler(request):
-        return web.json_response({'ok': True})
+        return web.json_response({"ok": True})
 
     app = web.Application()
-    app.router.add_post('/', handler)
+    app.router.add_post("/", handler)
 
     trace_config_ctx = mock.Mock()
     trace_request_ctx = {}
-    body = 'This is request body'
+    body = "This is request body"
     gathered_req_body = BytesIO()
     gathered_res_body = BytesIO()
-    on_request_start = mock.Mock(side_effect=asyncio.coroutine(mock.Mock()))
-    on_request_redirect = mock.Mock(side_effect=asyncio.coroutine(mock.Mock()))
-    on_request_end = mock.Mock(side_effect=asyncio.coroutine(mock.Mock()))
+    on_request_start = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
+    on_request_redirect = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
+    on_request_end = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
 
     async def on_request_chunk_sent(session, context, params):
         gathered_req_body.write(params.chunk)
@@ -552,7 +571,8 @@ async def test_request_tracing(loop, aiohttp_client) -> None:
     session = await aiohttp_client(app, trace_configs=[trace_config])
 
     async with session.post(
-            '/', data=body, trace_request_ctx=trace_request_ctx) as resp:
+        "/", data=body, trace_request_ctx=trace_request_ctx
+    ) as resp:
 
         await resp.json()
 
@@ -560,33 +580,26 @@ async def test_request_tracing(loop, aiohttp_client) -> None:
             session.session,
             trace_config_ctx,
             aiohttp.TraceRequestStartParams(
-                hdrs.METH_POST,
-                session.make_url('/'),
-                CIMultiDict()
-            )
+                hdrs.METH_POST, session.make_url("/"), CIMultiDict()
+            ),
         )
 
         on_request_end.assert_called_once_with(
             session.session,
             trace_config_ctx,
             aiohttp.TraceRequestEndParams(
-                hdrs.METH_POST,
-                session.make_url('/'),
-                CIMultiDict(),
-                resp
-            )
+                hdrs.METH_POST, session.make_url("/"), CIMultiDict(), resp
+            ),
         )
         assert not on_request_redirect.called
-        assert gathered_req_body.getvalue() == body.encode('utf8')
-        assert gathered_res_body.getvalue() == json.dumps(
-            {'ok': True}).encode('utf8')
+        assert gathered_req_body.getvalue() == body.encode("utf8")
+        assert gathered_res_body.getvalue() == json.dumps({"ok": True}).encode("utf8")
 
 
-async def test_request_tracing_exception(loop) -> None:
-    on_request_end = mock.Mock(side_effect=asyncio.coroutine(mock.Mock()))
-    on_request_exception = mock.Mock(
-        side_effect=asyncio.coroutine(mock.Mock())
-    )
+async def test_request_tracing_exception() -> None:
+    loop = asyncio.get_event_loop()
+    on_request_end = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
+    on_request_exception = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
 
     trace_config = aiohttp.TraceConfig()
     trace_config.on_request_end.append(on_request_end)
@@ -594,17 +607,18 @@ async def test_request_tracing_exception(loop) -> None:
 
     with mock.patch("aiohttp.client.TCPConnector.connect") as connect_patched:
         error = Exception()
-        f = loop.create_future()
-        f.set_exception(error)
-        connect_patched.return_value = f
+        if sys.version_info >= (3, 8, 1):
+            connect_patched.side_effect = error
+        else:
+            loop = asyncio.get_event_loop()
+            f = loop.create_future()
+            f.set_exception(error)
+            connect_patched.return_value = f
 
-        session = aiohttp.ClientSession(
-            loop=loop,
-            trace_configs=[trace_config]
-        )
+        session = aiohttp.ClientSession(loop=loop, trace_configs=[trace_config])
 
         try:
-            await session.get('http://example.com')
+            await session.get("http://example.com")
         except Exception:
             pass
 
@@ -612,22 +626,18 @@ async def test_request_tracing_exception(loop) -> None:
             session,
             mock.ANY,
             aiohttp.TraceRequestExceptionParams(
-                hdrs.METH_GET,
-                URL("http://example.com"),
-                CIMultiDict(),
-                error
-            )
+                hdrs.METH_GET, URL("http://example.com"), CIMultiDict(), error
+            ),
         )
         assert not on_request_end.called
 
 
 async def test_request_tracing_interpose_headers(loop, aiohttp_client) -> None:
-
     async def handler(request):
         return web.Response()
 
     app = web.Application()
-    app.router.add_get('/', handler)
+    app.router.add_get("/", handler)
 
     class MyClientRequest(ClientRequest):
         headers = None
@@ -636,35 +646,29 @@ async def test_request_tracing_interpose_headers(loop, aiohttp_client) -> None:
             super(MyClientRequest, self).__init__(*args, **kwargs)
             MyClientRequest.headers = self.headers
 
-    async def new_headers(
-            session,
-            trace_config_ctx,
-            data):
-        data.headers['foo'] = 'bar'
+    async def new_headers(session, trace_config_ctx, data):
+        data.headers["foo"] = "bar"
 
     trace_config = aiohttp.TraceConfig()
     trace_config.on_request_start.append(new_headers)
 
     session = await aiohttp_client(
-        app,
-        request_class=MyClientRequest,
-        trace_configs=[trace_config]
+        app, request_class=MyClientRequest, trace_configs=[trace_config]
     )
 
-    await session.get('/')
-    assert MyClientRequest.headers['foo'] == 'bar'
+    await session.get("/")
+    assert MyClientRequest.headers["foo"] == "bar"
 
 
-@pytest.mark.skipif(not PY_36,
-                    reason="Python 3.6+ required")
+@pytest.mark.skipif(not PY_36, reason="Python 3.6+ required")
 def test_client_session_inheritance() -> None:
     with pytest.warns(DeprecationWarning):
+
         class A(ClientSession):
             pass
 
 
-@pytest.mark.skipif(not DEBUG,
-                    reason="The check is applied in DEBUG mode only")
+@pytest.mark.skipif(not DEBUG, reason="The check is applied in DEBUG mode only")
 async def test_client_session_custom_attr(loop) -> None:
     session = ClientSession(loop=loop)
     with pytest.warns(DeprecationWarning):
@@ -676,21 +680,28 @@ async def test_client_session_timeout_args(loop) -> None:
     assert session1._timeout == client.DEFAULT_TIMEOUT
 
     with pytest.warns(DeprecationWarning):
-        session2 = ClientSession(loop=loop,
-                                 read_timeout=20*60,
-                                 conn_timeout=30*60)
-    assert session2._timeout == client.ClientTimeout(total=20*60,
-                                                     connect=30*60)
+        session2 = ClientSession(loop=loop, read_timeout=20 * 60, conn_timeout=30 * 60)
+    assert session2._timeout == client.ClientTimeout(total=20 * 60, connect=30 * 60)
 
     with pytest.raises(ValueError):
-        ClientSession(loop=loop,
-                      timeout=client.ClientTimeout(total=10*60),
-                      read_timeout=20*60)
+        ClientSession(
+            loop=loop, timeout=client.ClientTimeout(total=10 * 60), read_timeout=20 * 60
+        )
 
     with pytest.raises(ValueError):
-        ClientSession(loop=loop,
-                      timeout=client.ClientTimeout(total=10 * 60),
-                      conn_timeout=30 * 60)
+        ClientSession(
+            loop=loop, timeout=client.ClientTimeout(total=10 * 60), conn_timeout=30 * 60
+        )
+
+
+async def test_client_session_timeout_default_args(loop) -> None:
+    session1 = ClientSession()
+    assert session1.timeout == client.DEFAULT_TIMEOUT
+
+
+async def test_client_session_timeout_argument() -> None:
+    session = ClientSession(timeout=500)
+    assert session.timeout == 500
 
 
 async def test_requote_redirect_url_default() -> None:
