@@ -1,4 +1,4 @@
-"""Tests for streams.py"""
+# Tests for streams.py
 
 import abc
 import asyncio
@@ -13,18 +13,18 @@ import pytest
 
 from aiohttp import streams
 
-DATA = b'line1\nline2\nline3\n'
+DATA = b"line1\nline2\nline3\n"
 
 
 def chunkify(seq, n):
     for i in range(0, len(seq), n):
-        yield seq[i:i+n]
+        yield seq[i : i + n]
 
 
 async def create_stream():
     loop = asyncio.get_event_loop()
     protocol = mock.Mock(_reading_paused=False)
-    stream = streams.StreamReader(protocol, loop=loop)
+    stream = streams.StreamReader(protocol, 2 ** 16, loop=loop)
     stream.feed_data(DATA)
     stream.feed_eof()
     return stream
@@ -36,7 +36,7 @@ def protocol():
 
 
 MEMLEAK_SKIP_TYPES = (
-    *(getattr(types, name) for name in types.__all__ if name.endswith('Type')),
+    *(getattr(types, name) for name in types.__all__ if name.endswith("Type")),
     mock.Mock,
     abc.ABCMeta,
 )
@@ -69,23 +69,23 @@ def get_memory_usage(obj):
 
 class TestStreamReader:
 
-    DATA = b'line1\nline2\nline3\n'
+    DATA = b"line1\nline2\nline3\n"
 
     def _make_one(self, *args, **kwargs):
-        return streams.StreamReader(mock.Mock(_reading_paused=False),
-                                    *args, **kwargs)
+        kwargs.setdefault("limit", 2 ** 16)
+        return streams.StreamReader(mock.Mock(_reading_paused=False), *args, **kwargs)
 
     async def test_create_waiter(self) -> None:
         loop = asyncio.get_event_loop()
         stream = self._make_one(loop=loop)
         stream._waiter = loop.create_future
         with pytest.raises(RuntimeError):
-            await stream._wait('test')
+            await stream._wait("test")
 
     def test_ctor_global_loop(self) -> None:
         loop = asyncio.new_event_loop()
         asyncio.set_event_loop(loop)
-        stream = streams.StreamReader(mock.Mock(_reading_paused=False))
+        stream = streams.StreamReader(mock.Mock(_reading_paused=False), 2 ** 16)
 
         assert stream._loop is loop
 
@@ -93,13 +93,13 @@ class TestStreamReader:
         stream = self._make_one()
         assert not stream.at_eof()
 
-        stream.feed_data(b'some data\n')
+        stream.feed_data(b"some data\n")
         assert not stream.at_eof()
 
         await stream.readline()
         assert not stream.at_eof()
 
-        stream.feed_data(b'some data\n')
+        stream.feed_data(b"some data\n")
         stream.feed_eof()
         await stream.readline()
         assert stream.at_eof()
@@ -129,11 +129,11 @@ class TestStreamReader:
 
     async def test_feed_empty_data(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'')
+        stream.feed_data(b"")
         stream.feed_eof()
 
         data = await stream.read()
-        assert b'' == data
+        assert b"" == data
 
     async def test_feed_nonempty_data(self) -> None:
         stream = self._make_one()
@@ -149,7 +149,7 @@ class TestStreamReader:
         stream.feed_data(self.DATA)
 
         data = await stream.read(0)
-        assert b'' == data
+        assert b"" == data
 
         stream.feed_eof()
         data = await stream.read()
@@ -163,6 +163,7 @@ class TestStreamReader:
 
         def cb():
             stream.feed_data(self.DATA)
+
         loop.call_soon(cb)
 
         data = await read_task
@@ -170,41 +171,41 @@ class TestStreamReader:
 
         stream.feed_eof()
         data = await stream.read()
-        assert b'' == data
+        assert b"" == data
 
     async def test_read_line_breaks(self) -> None:
         # Read bytes without line breaks.
         stream = self._make_one()
-        stream.feed_data(b'line1')
-        stream.feed_data(b'line2')
+        stream.feed_data(b"line1")
+        stream.feed_data(b"line2")
 
         data = await stream.read(5)
-        assert b'line1' == data
+        assert b"line1" == data
 
         data = await stream.read(5)
-        assert b'line2' == data
+        assert b"line2" == data
 
     async def test_read_all(self) -> None:
         # Read all available buffered bytes
         stream = self._make_one()
-        stream.feed_data(b'line1')
-        stream.feed_data(b'line2')
+        stream.feed_data(b"line1")
+        stream.feed_data(b"line2")
         stream.feed_eof()
 
         data = await stream.read()
-        assert b'line1line2' == data
+        assert b"line1line2" == data
 
     async def test_read_up_to(self) -> None:
         # Read available buffered bytes up to requested amount
         stream = self._make_one()
-        stream.feed_data(b'line1')
-        stream.feed_data(b'line2')
+        stream.feed_data(b"line1")
+        stream.feed_data(b"line2")
 
         data = await stream.read(8)
-        assert b'line1lin' == data
+        assert b"line1lin" == data
 
         data = await stream.read(8)
-        assert b'e2' == data
+        assert b"e2" == data
 
     async def test_read_eof(self) -> None:
         loop = asyncio.get_event_loop()
@@ -214,20 +215,21 @@ class TestStreamReader:
 
         def cb():
             stream.feed_eof()
+
         loop.call_soon(cb)
 
         data = await read_task
-        assert b'' == data
+        assert b"" == data
 
         data = await stream.read()
-        assert data == b''
+        assert data == b""
 
     async def test_read_eof_infinite(self) -> None:
         # Read bytes.
         stream = self._make_one()
         stream.feed_eof()
 
-        with mock.patch('aiohttp.streams.internal_logger') as internal_logger:
+        with mock.patch("aiohttp.streams.internal_logger") as internal_logger:
             await stream.read()
             await stream.read()
             await stream.read()
@@ -241,14 +243,14 @@ class TestStreamReader:
         stream = self._make_one()
         stream.feed_eof()
 
-        with mock.patch('aiohttp.streams.internal_logger') as internal_logger:
+        with mock.patch("aiohttp.streams.internal_logger") as internal_logger:
             await stream.read()
             await stream.read()
             await stream.read()
             await stream.read()
             await stream.read()
         with pytest.warns(DeprecationWarning):
-            stream.unread_data(b'data')
+            stream.unread_data(b"data")
         await stream.read()
         await stream.read()
         assert not internal_logger.warning.called
@@ -260,23 +262,24 @@ class TestStreamReader:
         read_task = loop.create_task(stream.read(-1))
 
         def cb():
-            stream.feed_data(b'chunk1\n')
-            stream.feed_data(b'chunk2')
+            stream.feed_data(b"chunk1\n")
+            stream.feed_data(b"chunk2")
             stream.feed_eof()
+
         loop.call_soon(cb)
 
         data = await read_task
-        assert b'chunk1\nchunk2' == data
+        assert b"chunk1\nchunk2" == data
 
         data = await stream.read()
-        assert b'' == data
+        assert b"" == data
 
     async def test_read_exception(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'line\n')
+        stream.feed_data(b"line\n")
 
         data = await stream.read(2)
-        assert b'li' == data
+        assert b"li" == data
 
         stream.set_exception(ValueError())
         with pytest.raises(ValueError):
@@ -287,36 +290,37 @@ class TestStreamReader:
         # Read one line. 'readline' will need to wait for the data
         # to come from 'cb'
         stream = self._make_one()
-        stream.feed_data(b'chunk1 ')
+        stream.feed_data(b"chunk1 ")
         read_task = loop.create_task(stream.readline())
 
         def cb():
-            stream.feed_data(b'chunk2 ')
-            stream.feed_data(b'chunk3 ')
-            stream.feed_data(b'\n chunk4')
+            stream.feed_data(b"chunk2 ")
+            stream.feed_data(b"chunk3 ")
+            stream.feed_data(b"\n chunk4")
+
         loop.call_soon(cb)
 
         line = await read_task
-        assert b'chunk1 chunk2 chunk3 \n' == line
+        assert b"chunk1 chunk2 chunk3 \n" == line
 
         stream.feed_eof()
         data = await stream.read()
-        assert b' chunk4' == data
+        assert b" chunk4" == data
 
     async def test_readline_limit_with_existing_data(self) -> None:
         # Read one line. The data is in StreamReader's buffer
         # before the event loop is run.
 
         stream = self._make_one(limit=2)
-        stream.feed_data(b'li')
-        stream.feed_data(b'ne1\nline2\n')
+        stream.feed_data(b"li")
+        stream.feed_data(b"ne1\nline2\n")
 
         with pytest.raises(ValueError):
             await stream.readline()
         # The buffer should contain the remaining data after exception
         stream.feed_eof()
         data = await stream.read()
-        assert b'line2\n' == data
+        assert b"line2\n" == data
 
     async def test_readline_limit(self) -> None:
         loop = asyncio.get_event_loop()
@@ -325,16 +329,17 @@ class TestStreamReader:
         stream = self._make_one(limit=4)
 
         def cb():
-            stream.feed_data(b'chunk1')
-            stream.feed_data(b'chunk2\n')
-            stream.feed_data(b'chunk3\n')
+            stream.feed_data(b"chunk1")
+            stream.feed_data(b"chunk2\n")
+            stream.feed_data(b"chunk3\n")
             stream.feed_eof()
+
         loop.call_soon(cb)
 
         with pytest.raises(ValueError):
             await stream.readline()
         data = await stream.read()
-        assert b'chunk3\n' == data
+        assert b"chunk3\n" == data
 
     async def test_readline_nolimit_nowait(self) -> None:
         # All needed data for the first 'readline' call will be
@@ -344,26 +349,26 @@ class TestStreamReader:
         stream.feed_data(self.DATA[6:])
 
         line = await stream.readline()
-        assert b'line1\n' == line
+        assert b"line1\n" == line
 
         stream.feed_eof()
         data = await stream.read()
-        assert b'line2\nline3\n' == data
+        assert b"line2\nline3\n" == data
 
     async def test_readline_eof(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'some data')
+        stream.feed_data(b"some data")
         stream.feed_eof()
 
         line = await stream.readline()
-        assert b'some data' == line
+        assert b"some data" == line
 
     async def test_readline_empty_eof(self) -> None:
         stream = self._make_one()
         stream.feed_eof()
 
         line = await stream.readline()
-        assert b'' == line
+        assert b"" == line
 
     async def test_readline_read_byte_count(self) -> None:
         stream = self._make_one()
@@ -372,18 +377,18 @@ class TestStreamReader:
         await stream.readline()
 
         data = await stream.read(7)
-        assert b'line2\nl' == data
+        assert b"line2\nl" == data
 
         stream.feed_eof()
         data = await stream.read()
-        assert b'ine3\n' == data
+        assert b"ine3\n" == data
 
     async def test_readline_exception(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'line\n')
+        stream.feed_data(b"line\n")
 
         data = await stream.readline()
-        assert b'line\n' == data
+        assert b"line\n" == data
 
         stream.set_exception(ValueError())
         with pytest.raises(ValueError):
@@ -395,7 +400,7 @@ class TestStreamReader:
         stream.feed_data(self.DATA)
 
         data = await stream.readexactly(0)
-        assert b'' == data
+        assert b"" == data
         stream.feed_eof()
         data = await stream.read()
         assert self.DATA == data
@@ -404,7 +409,7 @@ class TestStreamReader:
         stream.feed_data(self.DATA)
 
         data = await stream.readexactly(-1)
-        assert b'' == data
+        assert b"" == data
         stream.feed_eof()
         data = await stream.read()
         assert self.DATA == data
@@ -421,6 +426,7 @@ class TestStreamReader:
             stream.feed_data(self.DATA)
             stream.feed_data(self.DATA)
             stream.feed_data(self.DATA)
+
         loop.call_soon(cb)
 
         data = await read_task
@@ -440,23 +446,23 @@ class TestStreamReader:
         def cb():
             stream.feed_data(self.DATA)
             stream.feed_eof()
+
         loop.call_soon(cb)
 
         with pytest.raises(asyncio.IncompleteReadError) as cm:
             await read_task
         assert cm.value.partial == self.DATA
         assert cm.value.expected == n
-        assert (str(cm.value) ==
-                '18 bytes read on a total of 36 expected bytes')
+        assert str(cm.value) == "18 bytes read on a total of 36 expected bytes"
         data = await stream.read()
-        assert b'' == data
+        assert b"" == data
 
     async def test_readexactly_exception(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'line\n')
+        stream.feed_data(b"line\n")
 
         data = await stream.readexactly(2)
-        assert b'li' == data
+        assert b"li" == data
 
         stream.set_exception(ValueError())
         with pytest.raises(ValueError):
@@ -464,51 +470,51 @@ class TestStreamReader:
 
     async def test_unread_data(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'line1')
-        stream.feed_data(b'line2')
-        stream.feed_data(b'onemoreline')
+        stream.feed_data(b"line1")
+        stream.feed_data(b"line2")
+        stream.feed_data(b"onemoreline")
 
         data = await stream.read(5)
-        assert b'line1' == data
+        assert b"line1" == data
 
         with pytest.warns(DeprecationWarning):
             stream.unread_data(data)
 
         data = await stream.read(5)
-        assert b'line1' == data
+        assert b"line1" == data
 
         data = await stream.read(4)
-        assert b'line' == data
+        assert b"line" == data
 
         with pytest.warns(DeprecationWarning):
-            stream.unread_data(b'line1line')
+            stream.unread_data(b"line1line")
 
-        data = b''
+        data = b""
         while len(data) < 10:
             data += await stream.read(10)
-        assert b'line1line2' == data
+        assert b"line1line2" == data
 
         data = await stream.read(7)
-        assert b'onemore' == data
+        assert b"onemore" == data
 
         with pytest.warns(DeprecationWarning):
             stream.unread_data(data)
 
-        data = b''
+        data = b""
         while len(data) < 11:
             data += await stream.read(11)
-        assert b'onemoreline' == data
+        assert b"onemoreline" == data
 
         with pytest.warns(DeprecationWarning):
-            stream.unread_data(b'line')
+            stream.unread_data(b"line")
         data = await stream.read(4)
-        assert b'line' == data
+        assert b"line" == data
 
         stream.feed_eof()
         with pytest.warns(DeprecationWarning):
-            stream.unread_data(b'at_eof')
+            stream.unread_data(b"at_eof")
         data = await stream.read(6)
-        assert b'at_eof' == data
+        assert b"at_eof" == data
 
     async def test_exception(self) -> None:
         stream = self._make_one()
@@ -544,7 +550,7 @@ class TestStreamReader:
         t.cancel()
         await asyncio.sleep(0)
         # The following line fails if set_exception() isn't careful.
-        stream.set_exception(RuntimeError('message'))
+        stream.set_exception(RuntimeError("message"))
         await asyncio.sleep(0)
         assert stream._waiter is None
 
@@ -552,13 +558,13 @@ class TestStreamReader:
         loop = asyncio.get_event_loop()
         stream = self._make_one()
         read_task = loop.create_task(stream.readany())
-        loop.call_soon(stream.feed_data, b'chunk1\n')
+        loop.call_soon(stream.feed_data, b"chunk1\n")
 
         data = await read_task
-        assert b'chunk1\n' == data
+        assert b"chunk1\n" == data
         stream.feed_eof()
         data = await stream.read()
-        assert b'' == data
+        assert b"" == data
 
     async def test_readany_empty_eof(self) -> None:
         loop = asyncio.get_event_loop()
@@ -568,14 +574,14 @@ class TestStreamReader:
 
         data = await read_task
 
-        assert b'' == data
+        assert b"" == data
 
     async def test_readany_exception(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'line\n')
+        stream.feed_data(b"line\n")
 
         data = await stream.readany()
-        assert b'line\n' == data
+        assert b"line\n" == data
 
         stream.set_exception(ValueError())
         with pytest.raises(ValueError):
@@ -583,28 +589,28 @@ class TestStreamReader:
 
     async def test_read_nowait(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'line1\nline2\n')
+        stream.feed_data(b"line1\nline2\n")
 
-        assert stream.read_nowait() == b'line1\nline2\n'
-        assert stream.read_nowait() == b''
+        assert stream.read_nowait() == b"line1\nline2\n"
+        assert stream.read_nowait() == b""
         stream.feed_eof()
         data = await stream.read()
-        assert b'' == data
+        assert b"" == data
 
     async def test_read_nowait_n(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'line1\nline2\n')
+        stream.feed_data(b"line1\nline2\n")
 
-        assert stream.read_nowait(4) == b'line'
-        assert stream.read_nowait() == b'1\nline2\n'
-        assert stream.read_nowait() == b''
+        assert stream.read_nowait(4) == b"line"
+        assert stream.read_nowait() == b"1\nline2\n"
+        assert stream.read_nowait() == b""
         stream.feed_eof()
         data = await stream.read()
-        assert b'' == data
+        assert b"" == data
 
     async def test_read_nowait_exception(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'line\n')
+        stream.feed_data(b"line\n")
         stream.set_exception(ValueError())
 
         with pytest.raises(ValueError):
@@ -613,7 +619,7 @@ class TestStreamReader:
     async def test_read_nowait_waiter(self) -> None:
         loop = asyncio.get_event_loop()
         stream = self._make_one()
-        stream.feed_data(b'line\n')
+        stream.feed_data(b"line\n")
         stream._waiter = loop.create_future()
 
         with pytest.raises(RuntimeError):
@@ -624,21 +630,22 @@ class TestStreamReader:
         stream = self._make_one()
 
         def cb():
-            stream.feed_data(b'chunk1')
-            stream.feed_data(b'chunk2')
+            stream.feed_data(b"chunk1")
+            stream.feed_data(b"chunk2")
             stream.feed_eof()
+
         loop.call_soon(cb)
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'chunk1' == data
+        assert b"chunk1" == data
         assert not end_of_chunk
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'chunk2' == data
+        assert b"chunk2" == data
         assert not end_of_chunk
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'' == data
+        assert b"" == data
         assert not end_of_chunk
 
     async def test_readchunk_wait_eof(self) -> None:
@@ -659,40 +666,40 @@ class TestStreamReader:
         stream = self._make_one()
 
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part1')
-        stream.feed_data(b'part2')
+        stream.feed_data(b"part1")
+        stream.feed_data(b"part2")
         stream.end_http_chunk_receiving()
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'part1part2' == data
+        assert b"part1part2" == data
         assert end_of_chunk
 
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part3')
+        stream.feed_data(b"part3")
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'part3' == data
+        assert b"part3" == data
         assert not end_of_chunk
 
         stream.end_http_chunk_receiving()
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'' == data
+        assert b"" == data
         assert end_of_chunk
 
         stream.feed_eof()
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'' == data
+        assert b"" == data
         assert not end_of_chunk
 
     async def test_readany_chunk_end_race(self) -> None:
         stream = self._make_one()
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part1')
+        stream.feed_data(b"part1")
 
         data = await stream.readany()
-        assert data == b'part1'
+        assert data == b"part1"
 
         loop = asyncio.get_event_loop()
         task = loop.create_task(stream.readany())
@@ -713,11 +720,11 @@ class TestStreamReader:
 
         stream.begin_http_chunk_receiving()
         # This SHOULD unblock the task actually.
-        stream.feed_data(b'part2')
+        stream.feed_data(b"part2")
         stream.end_http_chunk_receiving()
 
         data = await task
-        assert data == b'part2'
+        assert data == b"part2"
 
     async def test_end_chunk_receiving_without_begin(self) -> None:
         stream = self._make_one()
@@ -725,16 +732,15 @@ class TestStreamReader:
             stream.end_http_chunk_receiving()
 
     async def test_readchunk_with_unread(self) -> None:
-        """Test that stream.unread does not break controlled chunk receiving.
-        """
+        # Test that stream.unread does not break controlled chunk receiving.
         stream = self._make_one()
 
         # Send 2 chunks
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part1')
+        stream.feed_data(b"part1")
         stream.end_http_chunk_receiving()
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part2')
+        stream.feed_data(b"part2")
         stream.end_http_chunk_receiving()
 
         # Read only one chunk
@@ -742,74 +748,73 @@ class TestStreamReader:
 
         # Try to unread a part of the first chunk
         with pytest.warns(DeprecationWarning):
-            stream.unread_data(b'rt1')
+            stream.unread_data(b"rt1")
 
         # The end_of_chunk signal was already received for the first chunk,
         # so we receive up to the second one
         data, end_of_chunk = await stream.readchunk()
-        assert b'rt1part2' == data
+        assert b"rt1part2" == data
         assert end_of_chunk
 
         # Unread a part of the second chunk
         with pytest.warns(DeprecationWarning):
-            stream.unread_data(b'rt2')
+            stream.unread_data(b"rt2")
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'rt2' == data
+        assert b"rt2" == data
         # end_of_chunk was already received for this chunk
         assert not end_of_chunk
 
         stream.feed_eof()
         data, end_of_chunk = await stream.readchunk()
-        assert b'' == data
+        assert b"" == data
         assert not end_of_chunk
 
     async def test_readchunk_with_other_read_calls(self) -> None:
-        """Test that stream.readchunk works when other read calls are made on
-        the stream.
-        """
+        # Test that stream.readchunk works when other read calls are made on
+        # the stream.
         stream = self._make_one()
 
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part1')
+        stream.feed_data(b"part1")
         stream.end_http_chunk_receiving()
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part2')
+        stream.feed_data(b"part2")
         stream.end_http_chunk_receiving()
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part3')
+        stream.feed_data(b"part3")
         stream.end_http_chunk_receiving()
 
         data = await stream.read(7)
-        assert b'part1pa' == data
+        assert b"part1pa" == data
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'rt2' == data
+        assert b"rt2" == data
         assert end_of_chunk
 
         # Corner case between read/readchunk
         data = await stream.read(5)
-        assert b'part3' == data
+        assert b"part3" == data
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'' == data
+        assert b"" == data
         assert end_of_chunk
 
         stream.feed_eof()
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'' == data
+        assert b"" == data
         assert not end_of_chunk
 
     async def test_chunksplits_memory_leak(self) -> None:
-        """ Test for memory leak on chunksplits """
+        # Test for memory leak on chunksplits
         stream = self._make_one()
 
         N = 500
 
         # Warm-up variables
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'Y' * N)
+        stream.feed_data(b"Y" * N)
         stream.end_http_chunk_receiving()
         await stream.read(N)
 
@@ -818,7 +823,7 @@ class TestStreamReader:
         before = get_memory_usage(stream)
         for _ in range(N):
             stream.begin_http_chunk_receiving()
-            stream.feed_data(b'X')
+            stream.feed_data(b"X")
             stream.end_http_chunk_receiving()
         await stream.read(N)
         after = get_memory_usage(stream)
@@ -826,7 +831,7 @@ class TestStreamReader:
         assert abs(after - before) == 0
 
     async def test_read_empty_chunks(self) -> None:
-        """Test that feeding empty chunks does not break stream"""
+        # Test that feeding empty chunks does not break stream
         stream = self._make_one()
 
         # Simulate empty first chunk. This is significant special case
@@ -834,7 +839,7 @@ class TestStreamReader:
         stream.end_http_chunk_receiving()
 
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'ungzipped')
+        stream.feed_data(b"ungzipped")
         stream.end_http_chunk_receiving()
 
         # Possible when compression is enabled.
@@ -846,26 +851,25 @@ class TestStreamReader:
         stream.end_http_chunk_receiving()
 
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b' data')
+        stream.feed_data(b" data")
         stream.end_http_chunk_receiving()
 
         stream.feed_eof()
 
         data = await stream.read()
-        assert data == b'ungzipped data'
+        assert data == b"ungzipped data"
 
     async def test_readchunk_separate_http_chunk_tail(self) -> None:
-        """Test that stream.readchunk returns (b'', True) when end of
-        http chunk received after body
-        """
+        # Test that stream.readchunk returns (b'', True) when end of
+        # http chunk received after body
         loop = asyncio.get_event_loop()
         stream = self._make_one()
 
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part1')
+        stream.feed_data(b"part1")
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'part1' == data
+        assert b"part1" == data
         assert not end_of_chunk
 
         async def cb():
@@ -874,32 +878,32 @@ class TestStreamReader:
 
         loop.create_task(cb())
         data, end_of_chunk = await stream.readchunk()
-        assert b'' == data
+        assert b"" == data
         assert end_of_chunk
 
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part2')
+        stream.feed_data(b"part2")
         data, end_of_chunk = await stream.readchunk()
-        assert b'part2' == data
+        assert b"part2" == data
         assert not end_of_chunk
 
         stream.end_http_chunk_receiving()
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part3')
+        stream.feed_data(b"part3")
         stream.end_http_chunk_receiving()
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'' == data
+        assert b"" == data
         assert end_of_chunk
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'part3' == data
+        assert b"part3" == data
         assert end_of_chunk
 
         stream.begin_http_chunk_receiving()
-        stream.feed_data(b'part4')
+        stream.feed_data(b"part4")
         data, end_of_chunk = await stream.readchunk()
-        assert b'part4' == data
+        assert b"part4" == data
         assert not end_of_chunk
 
         async def cb():
@@ -909,11 +913,11 @@ class TestStreamReader:
 
         loop.create_task(cb())
         data, end_of_chunk = await stream.readchunk()
-        assert b'' == data
+        assert b"" == data
         assert end_of_chunk
 
         data, end_of_chunk = await stream.readchunk()
-        assert b'' == data
+        assert b"" == data
         assert not end_of_chunk
 
     async def test___repr__(self) -> None:
@@ -931,7 +935,7 @@ class TestStreamReader:
 
     async def test___repr__data(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'data')
+        stream.feed_data(b"data")
         assert "<StreamReader 4 bytes>" == repr(stream)
 
     async def test___repr__exception(self) -> None:
@@ -945,8 +949,7 @@ class TestStreamReader:
         loop = asyncio.get_event_loop()
         stream = self._make_one()
         stream._waiter = loop.create_future()
-        assert re.search(r"<StreamReader w=<Future pending[\S ]*>>",
-                         repr(stream))
+        assert re.search(r"<StreamReader w=<Future pending[\S ]*>>", repr(stream))
         stream._waiter.set_result(None)
         await stream._waiter
         stream._waiter = None
@@ -954,13 +957,13 @@ class TestStreamReader:
 
     async def test_unread_empty(self) -> None:
         stream = self._make_one()
-        stream.feed_data(b'line1')
+        stream.feed_data(b"line1")
         stream.feed_eof()
         with pytest.warns(DeprecationWarning):
-            stream.unread_data(b'')
+            stream.unread_data(b"")
 
         data = await stream.read(5)
-        assert b'line1' == data
+        assert b"line1" == data
         assert stream.at_eof()
 
 
@@ -969,16 +972,16 @@ async def test_empty_stream_reader() -> None:
     assert s.set_exception(ValueError()) is None
     assert s.exception() is None
     assert s.feed_eof() is None
-    assert s.feed_data(b'data') is None
+    assert s.feed_data(b"data") is None
     assert s.at_eof()
     assert (await s.wait_eof()) is None
-    assert await s.read() == b''
-    assert await s.readline() == b''
-    assert await s.readany() == b''
-    assert await s.readchunk() == (b'', True)
+    assert await s.read() == b""
+    assert await s.readline() == b""
+    assert await s.readany() == b""
+    assert await s.readchunk() == (b"", True)
     with pytest.raises(asyncio.IncompleteReadError):
         await s.readexactly(10)
-    assert s.read_nowait() == b''
+    assert s.read_nowait() == b""
 
 
 @pytest.fixture
@@ -987,7 +990,6 @@ async def buffer(loop):
 
 
 class TestDataQueue:
-
     def test_is_eof(self, buffer) -> None:
         assert not buffer.is_eof()
         buffer.feed_eof()
@@ -1015,6 +1017,7 @@ class TestDataQueue:
 
         def cb():
             buffer.feed_data(item, 1)
+
         loop.call_soon(cb)
 
         data = await buffer.read()
@@ -1025,6 +1028,7 @@ class TestDataQueue:
 
         def cb():
             buffer.feed_eof()
+
         loop.call_soon(cb)
 
         with pytest.raises(streams.EofStream):
@@ -1043,7 +1047,7 @@ class TestDataQueue:
         assert waiter.cancelled()
         assert buffer._waiter is None
 
-        buffer.feed_data(b'test', 4)
+        buffer.feed_data(b"test", 4)
         assert buffer._waiter is None
 
     async def test_read_until_eof(self, buffer) -> None:
@@ -1119,12 +1123,12 @@ class TestDataQueue:
 
 async def test_feed_data_waiters(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, loop=loop)
+    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
 
-    reader.feed_data(b'1')
-    assert list(reader._buffer) == [b'1']
+    reader.feed_data(b"1")
+    assert list(reader._buffer) == [b"1"]
     assert reader._size == 1
     assert reader.total_bytes == 1
 
@@ -1136,18 +1140,18 @@ async def test_feed_data_waiters(protocol) -> None:
 
 async def test_feed_data_completed_waiters(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, loop=loop)
+    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
     waiter = reader._waiter = loop.create_future()
 
     waiter.set_result(1)
-    reader.feed_data(b'1')
+    reader.feed_data(b"1")
 
     assert reader._waiter is None
 
 
 async def test_feed_eof_waiters(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, loop=loop)
+    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
 
@@ -1162,7 +1166,7 @@ async def test_feed_eof_waiters(protocol) -> None:
 
 async def test_feed_eof_cancelled(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, loop=loop)
+    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
 
@@ -1179,7 +1183,7 @@ async def test_feed_eof_cancelled(protocol) -> None:
 
 async def test_on_eof(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, loop=loop)
+    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
 
     on_eof = mock.Mock()
     reader.on_eof(on_eof)
@@ -1200,7 +1204,7 @@ async def test_on_eof_empty_reader() -> None:
 
 async def test_on_eof_exc_in_callback(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, loop=loop)
+    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
 
     on_eof = mock.Mock()
     on_eof.side_effect = ValueError
@@ -1224,7 +1228,7 @@ async def test_on_eof_exc_in_callback_empty_stream_reader() -> None:
 
 async def test_on_eof_eof_is_set(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, loop=loop)
+    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
     reader.feed_eof()
 
     on_eof = mock.Mock()
@@ -1235,7 +1239,7 @@ async def test_on_eof_eof_is_set(protocol) -> None:
 
 async def test_on_eof_eof_is_set_exception(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, loop=loop)
+    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
     reader.feed_eof()
 
     on_eof = mock.Mock()
@@ -1248,7 +1252,7 @@ async def test_on_eof_eof_is_set_exception(protocol) -> None:
 
 async def test_set_exception(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, loop=loop)
+    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
 
@@ -1263,7 +1267,7 @@ async def test_set_exception(protocol) -> None:
 
 async def test_set_exception_cancelled(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, loop=loop)
+    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
 
@@ -1281,7 +1285,7 @@ async def test_set_exception_cancelled(protocol) -> None:
 
 async def test_set_exception_eof_callbacks(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, loop=loop)
+    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
 
     on_eof = mock.Mock()
     reader.on_eof(on_eof)
@@ -1299,9 +1303,8 @@ async def test_stream_reader_lines() -> None:
 
 
 async def test_stream_reader_chunks_complete() -> None:
-    """Tests if chunked iteration works if the chunking works out
-    (i.e. the data is divisible by the chunk size)
-    """
+    # Tests if chunked iteration works if the chunking works out
+    # (i.e. the data is divisible by the chunk size)
     chunk_iter = chunkify(DATA, 9)
     async for data in (await create_stream()).iter_chunked(9):
         assert data == next(chunk_iter, None)
@@ -1309,7 +1312,7 @@ async def test_stream_reader_chunks_complete() -> None:
 
 
 async def test_stream_reader_chunks_incomplete() -> None:
-    """Tests if chunked iteration works if the last chunk is incomplete"""
+    # Tests if chunked iteration works if the last chunk is incomplete
     chunk_iter = chunkify(DATA, 8)
     async for data in (await create_stream()).iter_chunked(8):
         assert data == next(chunk_iter, None)
@@ -1317,7 +1320,7 @@ async def test_stream_reader_chunks_incomplete() -> None:
 
 
 async def test_data_queue_empty() -> None:
-    """Tests that async looping yields nothing if nothing is there"""
+    # Tests that async looping yields nothing if nothing is there
     loop = asyncio.get_event_loop()
     buffer = streams.DataQueue(loop)
     buffer.feed_eof()
@@ -1327,7 +1330,7 @@ async def test_data_queue_empty() -> None:
 
 
 async def test_data_queue_items() -> None:
-    """Tests that async looping yields objects identically"""
+    # Tests that async looping yields objects identically
     loop = asyncio.get_event_loop()
     buffer = streams.DataQueue(loop)
 
@@ -1343,23 +1346,21 @@ async def test_data_queue_items() -> None:
 
 
 async def test_stream_reader_iter_any() -> None:
-    it = iter([b'line1\nline2\nline3\n'])
+    it = iter([b"line1\nline2\nline3\n"])
     async for raw in (await create_stream()).iter_any():
         assert raw == next(it)
     pytest.raises(StopIteration, next, it)
 
 
 async def test_stream_reader_iter() -> None:
-    it = iter([b'line1\n',
-               b'line2\n',
-               b'line3\n'])
+    it = iter([b"line1\n", b"line2\n", b"line3\n"])
     async for raw in await create_stream():
         assert raw == next(it)
     pytest.raises(StopIteration, next, it)
 
 
 async def test_stream_reader_iter_chunks_no_chunked_encoding() -> None:
-    it = iter([b'line1\nline2\nline3\n'])
+    it = iter([b"line1\nline2\nline3\n"])
     async for data, end_of_chunk in (await create_stream()).iter_chunks():
         assert (data, end_of_chunk) == (next(it), False)
     pytest.raises(StopIteration, next, it)
@@ -1367,14 +1368,14 @@ async def test_stream_reader_iter_chunks_no_chunked_encoding() -> None:
 
 async def test_stream_reader_iter_chunks_chunked_encoding(protocol) -> None:
     loop = asyncio.get_event_loop()
-    stream = streams.StreamReader(protocol, loop=loop)
+    stream = streams.StreamReader(protocol, 2 ** 16, loop=loop)
     for line in DATA.splitlines(keepends=True):
         stream.begin_http_chunk_receiving()
         stream.feed_data(line)
         stream.end_http_chunk_receiving()
     stream.feed_eof()
 
-    it = iter([b'line1\n', b'line2\n', b'line3\n'])
+    it = iter([b"line1\n", b"line2\n", b"line3\n"])
     async for data, end_of_chunk in stream.iter_chunks():
         assert (data, end_of_chunk) == (next(it), True)
     pytest.raises(StopIteration, next, it)
